@@ -96,8 +96,8 @@ matattr <- function(free, est, constraint, mat, Ng, std.lv, wig, ...) {
     if (lvmat & length(ddd$sign) > 0) {
       lamfree <- ddd$free1
       lamfree2 <- ddd$free2
-      transtab <- cbind(sapply(lamfree, function(x) x[x != 0]),
-                        sapply(lamfree2, function(x) x[x != 0]))
+      transtab <- cbind(as.numeric(sapply(lamfree, function(x) x[x != 0])),
+                        as.numeric(sapply(lamfree2, function(x) x[x != 0])))
       lamsign <- ddd$sign
 
       for (i in 1:length(free2)) {
@@ -113,7 +113,7 @@ matattr <- function(free, est, constraint, mat, Ng, std.lv, wig, ...) {
 
             ## see whether any are equality constrained
             l1match <- match(lampar1, constraint$rhs, nomatch = 0L)
-            transconst <- transtab[match(constraint$lhs[l1match], transtab[,1]), 2*i]
+            transconst <- transtab[match(constraint$lhs[l1match], transtab[,1]), 2]
             lampar12[l1match != 0] <- as.numeric(transconst)
             if (all(lampar12 == 0)) { # ov converted to lv
               l1 <- 1
@@ -122,13 +122,13 @@ matattr <- function(free, est, constraint, mat, Ng, std.lv, wig, ...) {
               l1 <- lampar12[which(lampar12 %in% lamsign[,2])]
               ## for across-group equality constraint:
               if (length(l1) == 0) l1 <- lampar12[lampar12 != 0][1]
-              if (lamsign[l1,1] == 1) l1 <- lamsign[l1,2]
+              if (lamsign[l1[1],1] == 1) l1 <- lamsign[l1[1],2]
             }
 
             lampar2 <- lamfree[[i]][,fpar[j,1]]
             lampar22 <- lamfree2[[i]][,fpar[j,1]]
             l2match <- match(lampar2, constraint$rhs, nomatch = 0L)
-            transconst <- transtab[match(constraint$lhs[l2match], transtab[,1]), 2*i]
+            transconst <- transtab[match(constraint$lhs[l2match], transtab[,1]), 2]
             lampar22[l2match != 0] <- as.numeric(transconst)
             if (all(lampar22 == 0)) {
               l2 <- 1
@@ -136,7 +136,7 @@ matattr <- function(free, est, constraint, mat, Ng, std.lv, wig, ...) {
               lampar22 <- lampar22[lampar22 != 0]
               l2 <- lampar22[which(lampar22 %in% lamsign[,2])]
               if (length(l2) == 0) l2 <- lampar22[lampar22 != 0][1]
-              if (lamsign[l2,1] == 1) l2 <- lamsign[l2,2]
+              if (lamsign[l2[1],1] == 1) l2 <- lamsign[l2[1],2]
             }
 
             rowloc <- free2[[i]][fpar[j,1], fpar[j,2]]
@@ -185,9 +185,13 @@ lav2stanmarg <- function(lavobject, dp, n.chains, inits, wiggle=NULL, wiggle.sd=
   ## if not multilevel, this creates empty matrices to pass to stan for level 2
   ## if (!multilevel & level > 1) stop("blavaan ERROR: higher levels requested, but this is not a multilevel model.")
 
+  ## is this SAM?
+  dosam <- FALSE
+  if ("dosam" %in% names(mcmcextra)) dosam <- mcmcextra$dosam
+  
   ## data characteristics
   if (length(indat) == 0) {
-    dat <- lav2standata(lavobject)
+    dat <- lav2standata(lavobject, dosam = dosam)
     ## model
     if ("emiter" %in% names(mcmcextra$data)) {
       dat$emiter <- mcmcextra$data$emiter
@@ -551,7 +555,11 @@ lav2stanmarg <- function(lavobject, dp, n.chains, inits, wiggle=NULL, wiggle.sd=
     blkpsi <- all(sapply(blkinfo, function(x) x$isblk))
     blkse <- do.call("rbind", lapply(blkinfo, function(x) x$blkse))
     if (nrow(blkse) > 0) {
-      blkse <- blkse[(blkse[,3] == 1) & (blkse[,2] - blkse[,1] > 1), , drop = FALSE]
+      if (dosam) {
+        blkse <- blkse[blkse[,3] == 1, , drop = FALSE]
+      } else {
+        blkse <- blkse[(blkse[,3] == 1) & (blkse[,2] - blkse[,1] > 1), , drop = FALSE]
+      }
       blksizes <- blkse[,2] - blkse[,1] + 1
       ublksizes <- unique(blksizes)
       ublksizes <- ublksizes[order(ublksizes)]
@@ -568,7 +576,11 @@ lav2stanmarg <- function(lavobject, dp, n.chains, inits, wiggle=NULL, wiggle.sd=
       dat$psidims <- array(3, dim = 5)
       dat$blkse <- matrix(nrow = 0, ncol = 7)
     } else {
-      blkgrp <- rep(1:length(blkinfo), times = sapply(blkinfo, function(x) sum(x$blkse[,2] - x$blkse[,1] > 1)))
+      if (dosam) {
+        blkgrp <- rep(1:length(blkinfo), times = sapply(blkinfo, function(x) nrow(x$blkse)))
+      } else {
+        blkgrp <- rep(1:length(blkinfo), times = sapply(blkinfo, function(x) sum(x$blkse[,2] - x$blkse[,1] > 1)))
+      }
       arrayidx <- as.numeric(as.factor(ublksizes))
       dupsiz <- duplicated(blksizes)
       blkidx <- rep(NA, nrow(blkse))
@@ -632,7 +644,7 @@ lav2stanmarg <- function(lavobject, dp, n.chains, inits, wiggle=NULL, wiggle.sd=
     frnums <- sapply(fr, function(x) as.numeric(x[x != 0]))
     twsel <- lavpartable$free %in% frnums
     tmpwig <- lavpartable[twsel,'free'][which(lavpartable[twsel, 'plabel'] %in% wig)]
-    
+
     res <- matattr(fr, es, constrain, mat = "Psi_r", Ng, opts$std.lv, tmpwig,
                    free1 = free2$lambda, free2 = lyfree2, sign = dat$lam_y_sign,
                    dest = dest)
@@ -792,7 +804,11 @@ lav2stanmarg <- function(lavobject, dp, n.chains, inits, wiggle=NULL, wiggle.sd=
         }
         lptrow <- with(lavpartable, which(row == blkse[b,1] & col == (blkse[b,1] + 1) &
                                           group == blkse[b,4] & mat == "lvrho"))
-        blkse[b,7] <- as.numeric(gsub("(\\w+)\\(([^,]+)\\)", "\\2", lavpartable$prior[lptrow]))
+        if (!dosam | length(lptrow) > 0) {
+          blkse[b,7] <- as.numeric(gsub("(\\w+)\\(([^,]+)\\)", "\\2", lavpartable$prior[lptrow]))
+        } else {
+          blkse[b,7] <- 1
+        }
         lavpartable$prior[lkjrows] <- lavpartable$prior[lptrow]
       }
     }
@@ -859,6 +875,7 @@ lav2stanmarg <- function(lavobject, dp, n.chains, inits, wiggle=NULL, wiggle.sd=
           ini[[i]] <- c(ini[[i]], list(z_aug = z_aug))
         }
       }
+      if (dosam) ini[[i]]$Psi_sd_tmp <- ini[[i]]$Psi_sd_free
     }
   } else {
     ini <- NULL
@@ -1076,7 +1093,7 @@ coeffun_stanmarg <- function(lavpartable, lavfree, free2, lersdat, rsob, dmnames
 }
 
 ## organize information about the observed data for Stan (separately from model information)
-lav2standata <- function(lavobject) {
+lav2standata <- function(lavobject, dosam = FALSE) {
   dat <- list()
 
   Ng <- dat$Ng <- lavInspect(lavobject, 'ngroups')
@@ -1112,6 +1129,34 @@ lav2standata <- function(lavobject) {
   }
   allvars <- 1:nvar
 
+  if (dosam) {
+    lavmodel <- lavobject@Model
+
+    ## single group only
+    lvvars <- 1:NCOL(lavmodel@GLIST$lambda)
+    
+    dummy.ov.x.idx <- lavmodel@ov.x.dummy.ov.idx[[1]]
+    dummy.lv.x.idx <- lavmodel@ov.x.dummy.lv.idx[[1]]
+    dummy.ov.idx <- c(lavmodel@ov.y.dummy.ov.idx[[1]], dummy.ov.x.idx)
+    dummy.lv.idx <- c(lavmodel@ov.y.dummy.lv.idx[[1]], dummy.lv.x.idx)
+
+    dat$Ndum <- array(length(dummy.ov.idx), 1)
+    dum_ov_idx <- c(allvars[allvars %in% dummy.ov.idx],
+                    allvars[!(allvars %in% dummy.ov.idx)])
+    dat$dum_ov_idx <- array(dum_ov_idx, c(1, length(dum_ov_idx)))
+    dum_lv_idx <- c(lvvars[lvvars %in% dummy.lv.idx],
+                    lvvars[!(lvvars %in% dummy.lv.idx)])
+    dat$dum_lv_idx <- array(dum_lv_idx, c(1, length(dum_lv_idx)))
+
+    dat$Ndum_x <- array(length(dummy.ov.x.idx), 1)
+    dum_ov_x_idx <- c(allvars[allvars %in% dummy.ov.x.idx],
+                      allvars[!(allvars %in% dummy.ov.x.idx)])
+    dat$dum_ov_x_idx <- array(dum_ov_x_idx, c(1, length(dum_ov_x_idx)))
+    dum_lv_x_idx <- c(lvvars[lvvars %in% dummy.lv.x.idx],
+                      lvvars[!(lvvars %in% dummy.lv.x.idx)])
+    dat$dum_lv_x_idx <- array(dum_lv_x_idx, c(1, length(dum_lv_x_idx)))
+  }
+  
   ## lavobject@SampleStats@missing.flag is TRUE when missing='ml',
   ## regardless of whether data are missing
   misflag <- any(sapply(lavobject@Data@X, function(x) any(is.na(x))))
